package de.janschejbal.p2pbluetoothtest;

import java.io.File;
import java.util.Locale;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.text.Html;
import android.text.method.LinkMovementMethod;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.TextView.OnEditorActionListener;
import android.widget.Toast;
import de.janschejbal.p2pbluetoothtest.util.FileUtils;
import de.janschejbal.p2pbluetoothtest.util.PRNGFixes;

public class ConsentActivity extends Activity {
	// TODO ensure consent text reflects current app
	private static final String CONSENT_FILE = "consent_given";

	public static final String EXTRA_CONSENT_GIVEN = "CONSENT_GIVEN";
	
	private EditText consentInput;
	
    @Override
    public void onCreate(Bundle savedInstanceState) {
    	PRNGFixes.apply();
    	super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_consent);

        
        if (hasConsent(this)) {
        	findViewById(R.id.edit_consent).setVisibility(View.GONE);
        	findViewById(R.id.consent_info_2).setVisibility(View.GONE);
        	findViewById(R.id.consent_info_3).setVisibility(View.GONE);
        }
        
        
        TextView consentTextView = (TextView)findViewById(R.id.text_consent);
        String consentText = FileUtils.readString(getResources().openRawResource(R.raw.privacy_text));
        if (consentText == null) throw new RuntimeException("Failed to load consent text");
        consentTextView.setText(Html.fromHtml(consentText));
        consentTextView.setMovementMethod(LinkMovementMethod.getInstance());
        consentTextView.setFocusable(false);
        
        consentInput = (EditText)findViewById(R.id.edit_consent);
        consentInput.setOnEditorActionListener(new OnEditorActionListener() {
			public boolean onEditorAction(TextView v, int actionId, KeyEvent event) {
				return checkConsent();
			}
		});  
        
        Button okButton = (Button)findViewById(R.id.ok_consent);
        okButton.setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View v) {
				checkConsent();
			}
		});
    }
    
	private boolean consentEntered() {
		return consentInput.getText().toString().toLowerCase(Locale.US).matches("\\s*['\"]?einverstanden['\"]?\\s*");
	}
	
	private boolean checkConsent() {
		if (consentEntered()) {
			String consentInfo = System.currentTimeMillis() + " " + consentInput.getText().toString();
			if (FileUtils.writeString(ConsentActivity.this, CONSENT_FILE, consentInfo)) {
				Toast.makeText(ConsentActivity.this, "Einverstndnis gespeichert", Toast.LENGTH_LONG).show();						
			} else {
				Toast.makeText(ConsentActivity.this, "Konnte Einverstndnis nicht speichern", Toast.LENGTH_SHORT).show();
				EditText consentBox = (EditText)findViewById(R.id.edit_consent);
				consentBox.requestFocus();
			}
			finish();
			Intent intent = new Intent(ConsentActivity.this, MainActivity.class);
			intent.putExtra(EXTRA_CONSENT_GIVEN, true);
    		startActivity(intent);
			return true;
		}
		
		Toast.makeText(ConsentActivity.this, "Bitte \"Einverstanden\" eingeben", Toast.LENGTH_SHORT).show();
		return false;
	}
    
	private static boolean cachedConsent = false;
    public static boolean hasConsent(Context ctx) {
    	if (cachedConsent) return true;
    	if (new File(ctx.getFilesDir(), CONSENT_FILE).isFile()) {
    		cachedConsent = true;
    	}
    	return cachedConsent;
    }
    
    public static String getConsentInfo(Context ctx) {
    	if (!hasConsent(ctx)) return "--NO CONSENT FILE--";
    	try {
    		return FileUtils.readString(ctx, CONSENT_FILE);
    	} catch (Exception e) {
    		return "error";
    	}
    }

    
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        return false;
    }
}
