package de.janschejbal.p2pbluetoothtest.bluetooth;

import java.io.IOException;
import java.util.ArrayList;
import java.util.LinkedHashSet;

import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothServerSocket;
import android.bluetooth.BluetoothSocket;
import de.janschejbal.p2pbluetoothtest.Logger;
import de.janschejbal.p2pbluetoothtest.protocol.base.ConnectionFactory;
import de.janschejbal.p2pbluetoothtest.protocol.base.ProtocolConnection;
import de.janschejbal.p2pbluetoothtest.protocol.base.ProtocolConnection.CloseObserver;
import de.janschejbal.p2pbluetoothtest.util.FileUtils;

public class BluetoothServerThread extends Thread implements CloseObserver {
	private final static String TAG = BluetoothServerThread.class.getSimpleName();
	
	private Logger logger;

	private BluetoothServerSocket socket;
	private boolean run = true;
	
	private LinkedHashSet<ProtocolConnection> connections = new LinkedHashSet<ProtocolConnection>();

	private final ConnectionFactory connectionFactory;

	public BluetoothServerThread(ConnectionFactory connectionFactory, Logger logger) {
		this.connectionFactory = connectionFactory;
		this.logger = logger;
		try {
			this.socket = BluetoothAdapter.getDefaultAdapter().listenUsingInsecureRfcommWithServiceRecord(
					BluetoothCore.BT_SERVICE_NAME, BluetoothCore.BT_SERVICE_UUID);
			start();
		} catch (IOException e) {
			logger.log(TAG, "Could not create server socket", e);
		}
	}
	
	@Override
	public void run() {
		while (run) {
			BluetoothSocket conn;
			try {
				logger.log(TAG, "listening");				
				conn = socket.accept();
			} catch (IOException e) {
				logger.log(TAG, "Accept failed, dying", e);
				// TODO attempt restart?
				break; // die
			}
			logger.log(TAG, "connection accepted", conn.getRemoteDevice().getName());
			synchronized (connections) {
				connections.add(connectionFactory.getConnection(false, conn, logger, this));
			}
		}
		
		close();
	}
	
	@Override
	public void onConnectionClose(ProtocolConnection protocolConnection) {
		synchronized (connections) {
			connections.remove(protocolConnection);
		}
	}
	
	public boolean isConnectedTo(String address) {
		synchronized (connections) {
			for (ProtocolConnection conn : connections) {
				if (!conn.isClosed() && address.equals(conn.getRemoteDevice().getAddress())) return true;
			}
		}
		return false;
	}

	public void close() {
		synchronized (this) {
			if (!run) {
				logger.log(TAG, "already closed");
				return;
			} else {
				logger.log(TAG, "closing");
				run = false;
			}
		}
		FileUtils.closeSilent(socket); // automatically interrupts listen
		synchronized (connections) {
			ArrayList<ProtocolConnection> tmpConnections = new ArrayList<ProtocolConnection>(connections);
			connections.clear();
			for (ProtocolConnection conn : tmpConnections) {
				conn.close();
			}
		}
	}
}
