package de.janschejbal.p2pbluetoothtest.protocol.base;

import java.io.DataOutput;
import java.io.IOException;

public class SafeWriter implements DataOutput {

	private int length = -1;
	private int written = -1; // Writing nothing (not even the header) is OK too
	private DataOutput stream;
	
	public SafeWriter(DataOutput stream) {
		this.stream = stream;
	}
	
	public boolean verifyWriteLength() {
		return length == written;
	}

	private void checkWrite(int count) {
		if (written == -1) throw new RuntimeException("Writing message without header");
		written += count;
		if (written > length) throw new RuntimeException("Message overflow");
	}

	public void writeHeader(int tag, int length) throws IOException {
		this.length = length;
		this.written = 0;
		stream.writeInt(tag);
		stream.writeInt(length);
	}
	
	@Override
	public void write(int oneByte) throws IOException {
		checkWrite(1);
		stream.write(oneByte);
	}

	@Override
	public void writeByte(int val) throws IOException {
		checkWrite(1);
		stream.writeByte(val);
	}

	@Override
	public void write(byte[] bytes) throws IOException {
		checkWrite(bytes.length);
		stream.write(bytes);
	}

	@Override
	public void write(byte[] buffer, int offset, int count) throws IOException {
		checkWrite(count);
		stream.write(buffer, offset, count);
	}

	@Override
	public void writeBoolean(boolean val) throws IOException {
		checkWrite(1);
		stream.writeBoolean(val);
	}

	@Override
	public void writeChar(int val) throws IOException {
		checkWrite(2);
		stream.writeChar(val);
	}

	@Override
	public void writeShort(int val) throws IOException {
		checkWrite(2);
		stream.writeShort(val);
	}

	@Override
	public void writeInt(int val) throws IOException {
		checkWrite(4);
		stream.writeInt(val);
	}

	@Override
	public void writeLong(long val) throws IOException {
		checkWrite(8);
		stream.writeLong(val);
	}

	@Override
	public void writeFloat(float val) throws IOException {
		checkWrite(4);
		stream.writeFloat(val);
	}

	@Override
	public void writeDouble(double val) throws IOException {
		checkWrite(8);
		stream.writeDouble(val);
	}

	@Override
	public void writeBytes(String str) throws IOException {
		checkWrite(str.length()); // writeBytes writes one byte per string character
		stream.writeBytes(str);
	}

	@Override
	public void writeChars(String str) throws IOException {
		checkWrite(2*str.length()); // writeChars writes two bytes per string character
		stream.writeChars(str);
	}

	@Override
	public void writeUTF(String str) throws IOException {
		throw new UnsupportedOperationException("Not implemented");
	}

}
