package de.janschejbal.p2pbluetoothtest.broadcast;

import java.io.DataInput;
import java.io.IOException;
import java.io.Serializable;

import android.os.SystemClock;

public class TimestampedChannelState extends ChannelState implements Serializable {
	private static final long serialVersionUID = 1L;

	/**
	 * Timestamp indicating when this ChannelState was last changed (elapsedRealtime, used to reduce traffic)
	 */	
	private transient long lastChanged = SystemClock.elapsedRealtime();

	/**
	 * Absolute timestamp indicating when this ChannelState was last changed (used to prune dead channels)
	 * @param channelID
	 */
	private long lastChangedAbsolute = System.currentTimeMillis();
	
	public TimestampedChannelState(long channelID) {
		super(channelID);
	}

	public TimestampedChannelState(ChannelState orig) {
		super(orig);
	}

	public TimestampedChannelState(DataInput reader) throws IOException {
		super(reader);
	}
	
	@Override
	public boolean addMessage(int messageID) {
		if (super.addMessage(messageID)) {
			lastChanged = SystemClock.elapsedRealtime();
			lastChangedAbsolute = System.currentTimeMillis();
			return true;
		} else {
			return false;
		}
	}
	
	/**
	 * @return the elapsedRealtime when this ChannelState was last changed (used to reduce traffic)
	 */
	public long getLastChanged() {
		return lastChanged;
	}
	
	/**
	 * @return the absolute time (based on currentTimeMillis) when this ChannelState was last changed  (used to prune dead channels)
	 */
	public long getLastChangedAbsolute() {
		return lastChangedAbsolute;
	}
	
	private void readObject(java.io.ObjectInputStream in) throws IOException, ClassNotFoundException {
		in.defaultReadObject();
		lastChanged = SystemClock.elapsedRealtime();
	}

}
