package de.janschejbal.p2pbluetoothtest.protocol.broadcast;

import java.io.IOException;
import java.util.Random;

import de.janschejbal.p2pbluetoothtest.protocol.base.ProtocolException;
import de.janschejbal.p2pbluetoothtest.protocol.base.ProtocolMessage;
import de.janschejbal.p2pbluetoothtest.protocol.base.ProtocolMessageParser;
import de.janschejbal.p2pbluetoothtest.protocol.base.SafeReader;
import de.janschejbal.p2pbluetoothtest.protocol.base.SafeWriter;

public class PayloadMessage implements ProtocolMessage {
	private final static Random fastRandom = new Random();
	
	private final long channelId;
	private final int messageId;
	private final int payloadSize;
	
	/**
	 * Creates a new PayloadMessage with a simulated payload
	 * @param channelId channel ID of the simulated message
	 * @param messageId message ID of the simulated message
	 * @param payloadSize size of the simulated message
	 */
	public PayloadMessage(long channelId, int messageId, int payloadSize) {
		this.channelId = channelId;
		this.messageId = messageId;
		this.payloadSize = payloadSize;
	}

	@Override
	public void send(SafeWriter sink) throws IOException {
		sink.writeHeader(BroadcastProtocolConnection.MESSAGE_PAYLOAD, 8+4+payloadSize);
		sink.writeLong(channelId);
		sink.writeInt(messageId);
		
		byte[] payload = new byte[payloadSize];
		fastRandom.nextBytes(payload);
		sink.write(payload);
	}

	private static final ProtocolMessageParser<PayloadMessage> parser = new ProtocolMessageParser<PayloadMessage>() {
		@Override
		public PayloadMessage parseMessage(int tag, int length, SafeReader source) throws IOException {
			if (tag != BroadcastProtocolConnection.MESSAGE_PAYLOAD) throw new ProtocolException("Tag mismatch");
			
			long channelId = source.readLong();
			int messageId = source.readInt();
			int payloadSize = length - (8+4);
			source.chompOverhang();
			return new PayloadMessage(channelId, messageId, payloadSize);
		}
	};

	public static ProtocolMessageParser<PayloadMessage> getParser() {
		return parser;
	}

	public long getChannelId() {
		return channelId;
	}

	public int getMessageId() {
		return messageId;
	}

	public int getPayloadSize() {
		return payloadSize;
	}

}
