package de.janschejbal.p2pbluetoothtest.util;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.math.BigInteger;
import java.nio.ByteBuffer;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;

import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;

public class CryptoHelper {
	
	public final static SecureRandom random = new SecureRandom();
	
	public static String randomHexBytes(int byteCount) {
		byte[] buf = new byte[byteCount];
		random.nextBytes(buf);
		return bytesToHex(buf);
	}
	
	public static String bytesToHex(byte[] bytes) {
		StringBuilder buf = new StringBuilder(bytes.length*2);
		for (int i = 0; i < bytes.length; i++) buf.append(String.format("%02x", bytes[i] & 0xFF));
		return buf.toString();
	}
	
	public static String longToHex(long value) {
		return bytesToHex(ByteBuffer.allocate(8).putLong(value).array());
	}

	public static long hexToLong(String str) {
		if (str.length() != 16) throw new RuntimeException("Hex-Long must be exactly 16 hex-chars (8 byte)");
		return new BigInteger(str, 16).longValue();
	}

	public static String calculateHMAC(byte[] key, File file) throws NoSuchAlgorithmException,
			InvalidKeyException, FileNotFoundException, IOException {
		// Hash the file with HMAC
		Mac mac = Mac.getInstance("HmacSHA256");
		mac.init(new SecretKeySpec(key, "HmacSHA256"));
		FileInputStream filestream = new FileInputStream(file);
		byte[] buf = new byte[64*1024];
		int read;
		while ((read = filestream.read(buf)) > 0) {
			mac.update(buf, 0, read);
		}
		filestream.close();
		return CryptoHelper.bytesToHex(mac.doFinal());
	}
}
