package de.janschejbal.p2pbluetoothtest.protocol.broadcast;

import java.io.IOException;
import java.util.ArrayList;

import de.janschejbal.p2pbluetoothtest.broadcast.ChannelState;
import de.janschejbal.p2pbluetoothtest.protocol.base.ProtocolException;
import de.janschejbal.p2pbluetoothtest.protocol.base.ProtocolMessage;
import de.janschejbal.p2pbluetoothtest.protocol.base.ProtocolMessageParser;
import de.janschejbal.p2pbluetoothtest.protocol.base.SafeReader;
import de.janschejbal.p2pbluetoothtest.protocol.base.SafeWriter;

public class ChannelMessage implements ProtocolMessage {
	private final int tag;
	private final ArrayList<ChannelState> channels;
	
	/**
	 * Creates a new ChannelState message.
	 * The list is taken as-is and both list and the ChannelStates within must not be changed from outside. 
	 * @param states list of ChannelStates to include
	 */
	public ChannelMessage(int tag, ArrayList<ChannelState> states) {
		this.tag = tag;
		this.channels = states;
	}

	@Override
	public void send(SafeWriter sink) throws IOException {
		sink.writeHeader(tag, channels.size() * ChannelState.SERIALIZED_SIZE);
		for (ChannelState state : channels) {
			state.writeTo(sink);
		}
	}

	private static final ProtocolMessageParser<ChannelMessage> parser = new ProtocolMessageParser<ChannelMessage>() {
		@Override
		public ChannelMessage parseMessage(int tag, int length, SafeReader source) throws IOException {
			if (tag != BroadcastProtocolConnection.MESSAGE_STATE &&
					tag != BroadcastProtocolConnection.MESSAGE_REQUEST) {
				throw new ProtocolException("Tag mismatch");
			}
			
			if (length % ChannelState.SERIALIZED_SIZE != 0) {
				throw new ProtocolException("Invalid length in state message");
			}
			
			int count = length / ChannelState.SERIALIZED_SIZE;
			ArrayList<ChannelState> channels = new ArrayList<ChannelState>(count);
			for (int i = 0; i < count; i++) {
				channels.add(new ChannelState(source));
			}
			
			return new ChannelMessage(tag, channels);
		}
	};

	public static ProtocolMessageParser<ChannelMessage> getParser() {
		return parser;
	}

	public ArrayList<ChannelState> getChannels() {
		return channels;
	}

}
