package de.janschejbal.p2pbluetoothtest.protocol.broadcast;

import java.io.IOException;

import de.janschejbal.p2pbluetoothtest.protocol.base.ProtocolException;
import de.janschejbal.p2pbluetoothtest.protocol.base.ProtocolMessage;
import de.janschejbal.p2pbluetoothtest.protocol.base.ProtocolMessageParser;
import de.janschejbal.p2pbluetoothtest.protocol.base.SafeReader;
import de.janschejbal.p2pbluetoothtest.protocol.base.SafeWriter;

public class DebugMessage implements ProtocolMessage {
	private final String string;
	
	/**
	 * Creates a new DebugMessage
	 *  
	 * @param string the String to send to the other node (for it to log)
	 */
	public DebugMessage(String string) {
		this.string = string;
	}

	@Override
	public void send(SafeWriter sink) throws IOException {
		sink.writeHeader(BroadcastProtocolConnection.MESSAGE_DEBUG, string.length());
		sink.writeBytes(string);
	}

	private static final ProtocolMessageParser<DebugMessage> parser = new ProtocolMessageParser<DebugMessage>() {
		@Override
		public DebugMessage parseMessage(int tag, int length, SafeReader source) throws IOException {
			if (tag != BroadcastProtocolConnection.MESSAGE_DEBUG) throw new ProtocolException("Tag mismatch");
			
			byte[] buf = new byte[length];
			source.readFully(buf);
			StringBuilder strbuf = new StringBuilder(length);
			for (byte b : buf) {
				if (b < 127 && b > 31) {
					strbuf.appendCodePoint(b);
				} else {
					strbuf.append('~');
				}
			}
			return new DebugMessage(strbuf.toString());
		}
	};

	public static ProtocolMessageParser<DebugMessage> getParser() {
		return parser;
	}

	public String getString() {
		return string;
	}

}
